@@ -2,9 +2,13 @@
 //!
 //! provides functions to read and write bitcoin transactions from a txlog server
 
+use errors::TxLogErrors;
 use hex;
 use serde::{Deserialize, Serialize};
 
+mod errors;
+
+#[derive(Clone)]
 pub struct TxlogClient {
     pub url: String,
     secret: String,
@@ -31,62 +35,48 @@ pub fn new(url: String, secret: String) -> TxlogClient {
 
 impl TxlogClient {
     // fetch a raw transaction
-    pub async fn rawtx(&self, txid: &String) -> Option<Vec<u8>> {
+    pub async fn rawtx(&self, txid: &String) -> Result<Vec<u8>, TxLogErrors> {
         let client = reqwest::Client::new();
         let url = format!("{}/tx/{}/raw", &self.url, txid);
-        let response = match client
+        let response = client
             .get(&url)
             .header("Authorization", format!("Bearer {}", self.secret))
             .send()
-            .await
-        {
-            Err(_) => return None,
-            Ok(v) => v,
-        };
+            .await?;
 
         let body = match response.text().await {
-            Err(_) => return None,
+            Err(e) => return Err(TxLogErrors::BodyDecodeError(e)),
             Ok(v) => v,
         };
 
         let decoded = match hex::decode(&body) {
             Ok(v) => v,
-            Err(_) => return None,
+            Err(e) => return Err(TxLogErrors::HexDecode(e)),
         };
 
-        return Some(decoded);
+        return Ok(decoded);
     }
 
     // fetch transaction status
-    pub async fn tx(&self, txid: &String) -> Option<TxResponse> {
+    pub async fn tx(&self, txid: &String) -> Result<TxResponse, TxLogErrors> {
         let client = reqwest::Client::new();
         let url = format!("{}/tx/{}", &self.url, txid);
-        let response = match client
+        let response = client
             .get(&url)
             .header("Authorization", format!("Bearer {}", self.secret))
             .send()
-            .await
-        {
-            Err(e) => {
-                println!("{:?}", e);
-                return None;
-            }
+            .await?;
+        
+        let res = match response.json::<TxResponse>().await {
             Ok(v) => v,
+            Err(e) => return Err(TxLogErrors::BodyDecodeError(e)),
         };
 
-        let body = match response.json::<TxResponse>().await {
-            Err(e) => {
-                println!("{:?}", e);
-                return None;
-            }
-            Ok(v) => v,
-        };
-
-        return Some(body);
+        Ok(res)
     }
 
     // submit a transaction
-    pub async fn submit(&self, rawtx: Vec<u8>, metadata: Option<String>) -> Option<SubmitResponse> {
+    pub async fn submit(&self, rawtx: Vec<u8>, metadata: Option<String>) ->  Result<SubmitResponse, TxLogErrors> {
         let client = reqwest::Client::new();
         let mut url = format!("{}/tx", &self.url);
 
@@ -97,30 +87,24 @@ impl TxlogClient {
             None => (),
         }
 
-        let response = match client
+        let response = client
             .post(&url)
             .header("Authorization", format!("Bearer {}", self.secret))
             .header("Content-Type", "application/octet-stream")
             .body(rawtx)
             .send()
-            .await
-        {
-            Err(e) => {
-                println!("{:?}", e);
-                return None;
-            }
+            .await?;
+
+        let res = match response.json::<SubmitResponse>().await {
             Ok(v) => v,
+            Err(e) => return Err(TxLogErrors::BodyDecodeError(e)),
         };
 
-        let body = match response.json::<SubmitResponse>().await {
-            Err(e) => {
-                println!("{:?}", e);
-                return None;
-            }
-            Ok(v) => v,
-        };
+        if !res.success {
+            return Err(TxLogErrors::SubmitResponseFailed)
+        }
 
-        return Some(body);
+        Ok(res)
     }
 }
 
